unit BWClock;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls, BWTools;

type
  TBWClock = class(TCustomPanel)
  private
    FTimer : TTimer;
    FTZone : TTImeZoneInformation;
    FLocal : Boolean;
    FSeconds:boolean;
    FAMPM  : boolean;
    FPressed: boolean;
    function GetTimer : boolean;
    procedure SetTimer(value:boolean);
    procedure OnTimer(Sender:TObject);
    procedure WMLButtonDown( var Msg : TMessage ) ; message WM_LBUTTONDOWN ;
    procedure WMLButtonUp( var Msg : TWMLButtonUp ) ; message WM_LBUTTONUP ;
  protected
    { Protected declarations }
    procedure CreateTimer;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent); override;
    destructor Destroy; override;
    procedure Kick;
    property TimeZone : TTimeZoneInformation read FTZone write FTZone;
  published
    { Published declarations }
    property UseTimer : Boolean read GetTimer write SetTimer;
    property LocalTime: boolean read FLocal write FLocal;
    property ShowSeconds: boolean read FSeconds write FSeconds;
    property AM_PM : boolean read FAMPM write FAMPM;
    property Align;
    property BevelInner default bvLowered;
    property BevelOuter default bvRaised;
    property BevelWidth;
    property BorderWidth;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property Cursor;
    property Font;
    property Hint;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnDragOver;
    property OnDragDrop;
    property OnEndDrag;
    property OnStartDrag;
    property OnResize;
  end;

type PTimeZoneInfo = ^TTimeZoneInfo;
     TTimeZoneInfo = record
       Name        : string;
       DisplayName : string;
       TimeZone    : TTimeZoneInformation;
     end;

     PShortTZ = ^TShortTZ;
     TShortTZ = record
       Bias: Longint;
       StandardDate: TSystemTime;
       StandardBias: Longint;
       DaylightDate: TSystemTime;
       DaylightBias: Longint;
     end;

function RetrieveTimeZoneInfo(TimeZoneInfoList : TList):boolean;


// Uncomment this if you use the component without BWPack

// procedure Register;

{$R *.res}

implementation

constructor TBWClock.Create(AOwner:TComponent);
begin
  inherited;
  CreateTimer;
  BevelInner := bvLowered;
  BevelOuter := bvRaised;
end;

destructor TBWClock.Destroy;
begin
  if FTimer<>nil then
  begin
    FTimer.Free;
    FTimer:=nil;
  end;
  inherited;
end;

function IncTime(ATime: TDateTime; Hours, Minutes, Seconds,
  MSecs: Integer): TDateTime;
begin
  Result := ATime + (Hours div 24) + (((Hours mod 24) * 3600000 +
    Minutes * 60000 + Seconds * 1000 + MSecs) / MSecsPerDay);
end;

procedure TBWClock.Kick;
var LocalTime : TSystemTime;
    CurTime   : TSystemTime;
    ToDisplay : string;
    Buffer    : pchar;
    flags     : dword;
    format    : string;
    i : integer;

begin
  GetMem(Buffer,100);
  if FLocal then GetLocalTime(CurTime)
  else
  begin
    GetSystemTime(LocalTime);
    CurTime:=LocalTime;
    if IsWin95 then
    begin
      DateTimeToSystemTime(IncTime(SystemTimeToDateTime(LocalTime),0,-FTZone.Bias,0,0),CurTime);
    end
      else SystemTimeToTzSpecificLocalTime(@FTZone,LocalTime,CurTime);
  end;
  flags:=0;
  if not FAMPM then
  begin
    if ShowSeconds then format:='HH:mm:ss' else format:='HH:mm';
  end
  else
  begin
    if ShowSeconds then format:='h'':''mm'':''ss tt' else format:='h'':''mm tt';
  end;
  if FPressed then
    GetDateFormat(LOCALE_USER_DEFAULT,DATE_SHORTDATE, @CurTime, nil, Buffer, 100)
  else
    GetTimeFormat(LOCALE_USER_DEFAULT, Flags, @CurTime, pchar(format), Buffer, 100);
  i:=GetLastError;
  ToDisplay:=StrPas(Buffer);
  Caption:=ToDisplay;
  FreeMem(Buffer,100);
end;

procedure TBWClock.OnTimer(Sender:TObject);
begin
  Kick;
end;

procedure TBWClock.CreateTimer;
begin
  FTimer:=TTimer.Create(self);
  FTimer.Interval:=1000;
  FTimer.OnTimer:=OnTimer;
  FTimer.Enabled:=true;
end;

function TBWClock.GetTimer : boolean;
begin
  result:=Assigned(FTimer);
end;

procedure TBWClock.SetTimer(value:boolean);
begin
  if value<>Assigned(FTimer) then
  begin
    if value then CreateTimer else
    begin
      FTimer.Free;
      FTimer:=nil;
    end;
  end;
end;

procedure TBWClock.WMLButtonDown( var Msg : TMessage ) ;  { private }
begin
  inherited;
  FPressed:=true;
  Kick;
end ;  { WMLButtonDown }

procedure TBWClock.WMLButtonUp( var Msg : TWMLButtonUp ) ;  { private }
begin
  inherited;
  FPressed:=false;
  Kick;
end ;  { WMLButtonUp }

function RetrieveTimeZoneInfo;
type PByte  = ^byte;
var KeyStr,
    SubKeyStr : string;
    Key,
    SubKey : HKey;
    ptz    : PTimeZoneInfo;
    res    : DWORD;
    i      : integer;
    b1,b2  : pchar;
    bufsize: integer;
    atime  : TFileTime;
begin
  result:=false;
  try
    if IsWinNT
       then KeyStr:='Software\Microsoft\Windows NT\CurrentVersion\Time Zones'
       else KeyStr:='Software\Microsoft\Windows\CurrentVersion\Time Zones';
    if RegOpenKeyEx(HKEY_LOCAL_MACHINE,pchar(KeyStr), 0, KEY_READ,Key)<>ERROR_SUCCESS then exit;
    i:=0;
    repeat
      New(PTZ); //GetMem(PTZ, SizeOf(TTimeZoneInfo));
      GetMem(b1,100);
      BufSize:=100;
      res:=RegEnumKeyEx(Key,i,b1{key name},BufSize,nil,nil,nil,@atime);
      if res=ERROR_SUCCESS then
      begin
        PTZ^.Name:=StrPas(B1);
        SubKeyStr:=PTZ^.Name;
        if RegOpenKeyEx(Key, PChar(SubKeyStr), 0, KEY_READ, SubKey)=ERROR_SUCCESS then
        begin
          GetMem(b2,100);
          BufSize:=100;
          SetLength(PTZ^.DisplayName,100);
          if RegQueryValueEx(SubKey,'Display',nil,nil, @(byte(B2^)), @BufSize)=ERROR_SUCCESS then
             PTZ^.DisplayName:=StrPas(B2);
          BufSize:=100;
          if RegQueryValueEx(SubKey,'TZI',nil,nil,@(byte(B2^)),@BufSize)=ERROR_SUCCESS then
          begin
            PTZ^.TimeZone.Bias:=PShortTZ(B2)^.Bias;
            PTZ^.TimeZone.StandardDate:=PShortTZ(B2)^.StandardDate;
            PTZ^.TimeZone.StandardBias:=PShortTZ(B2)^.StandardBias;
            PTZ^.TimeZone.DaylightDate:=PShortTZ(B2)^.DaylightDate;
            PTZ^.TimeZone.DaylightBias:=PShortTZ(B2)^.DaylightBias;
          end;
          RegCloseKey(SubKey);
          FreeMem(b2,100);
        end;
        TimeZoneInfoList.Add(PTZ);
      end else Dispose(PTZ);
      inc(i);
      FreeMem(b1,100);
    until RES = ERROR_NO_MORE_ITEMS;
    RegCloseKey(Key);
    result:=true;
  except
    on E:EOutOfMemory do result:=false;
  end;
end;

//Uncomment this if you use the component without BWPack

{
procedure Register;
begin
  RegisterComponents('Wabbit''s', [TBWClock]);
end;
}

end.

