{+---------------------------------------+}
{|                                       |}
{|         Bugsy Wabbit's Tree           |}
{|           additional file             |}
{|              Shareware                |}
{|         by Eugene Mayevski            |}
{|                                       |}
{|         http://i.am/Bugsy             |}
{|             Bugsy@i.am                |}
{+---------------------------------------+}

unit frmItemsProp;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, DsgnIntf,
     Buttons, ExtCtrls, BWTree, frmItemCol, Dialogs;

type
  TItemsPropDlg = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    ApplyBtn: TButton;
    ItemsGB: TGroupBox;
    NewItemBtn: TButton;
    SubitemBtn: TButton;
    DeleteBtn: TButton;
    SaveBtn: TButton;
    LoadBtn: TButton;
    EditBtn: TButton;
    Tree: TBWTree;
    OpenDlg: TOpenDialog;
    SaveDlg: TSaveDialog;
    procedure DeleteBtnClick(Sender: TObject);
    procedure SubitemBtnClick(Sender: TObject);
    procedure TreeItemFocused(Sender: TObject);
    procedure NewItemBtnClick(Sender: TObject);
    procedure EditBtnClick(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure LoadBtnClick(Sender: TObject);
    procedure TreeStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure TreeDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure TreeDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FDragItem : TBWTreeItem;
//    procedure SetItems(value : TBWTreeItems);
//    function  GetItems:TBWTreeItems;
  public
    { Public declarations }
    DTree : TCustomBWTree;
  end;

  TBWTreeItemsProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue:string; override;
  end;

type TBWDragObject = class (TDragControlObject)
        function GetDragCursor(Accepted: Boolean; X, Y: Integer): TCursor; override;
     end;

var
  ItemsPropDlg: TItemsPropDlg;

implementation

{$R *.DFM}

function TBWDragObject.GetDragCursor(Accepted: Boolean; X, Y: Integer): TCursor;
begin
  if Control is TBWTree then
  begin
    if ((Control as TBWTree).GetItemAtY(Y)<>nil) or (Accepted) then
       Result := (Control as TBWTree).DragCursor else
       Result := crNoDrop;
  end else result:=inherited GetDragCursor(Accepted,X,Y);
end;

procedure TBWTreeItemsProperty.Edit;
var Editor: TItemsPropDlg;
begin
  Editor:=nil;
  try
    Editor:=TItemsPropDlg.Create(Application);
    Editor.DTree:=TCustomBWTree(GetComponent(0));
    Editor.Tree.Items:=Editor.DTree.Items;
    Editor.ShowModal;
  finally
    Editor.Free;
  end;
end;

function TBWTreeItemsProperty.GetAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TBWTreeItemsProperty.GetValue;
begin
  FmtStr(Result, '(%s)', [GetPropType^.Name]);
end;

// ===========================================================================

procedure TItemsPropDlg.DeleteBtnClick(Sender: TObject);
var Form : TCustomForm;
begin
  if Tree.ItemFocused<>nil then Tree.Items.DeleteItem(Tree.ItemFocused);
  Form := GetParentForm(DTree);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TItemsPropDlg.SubitemBtnClick(Sender: TObject);
var TSI:TBWTreeItem;
var Form : TCustomForm;
begin
  if Tree.ItemFocused<>nil then
  begin
    Tree.ItemFocused.Expanded:=true;
    TSI:=Tree.Items.AddItem(Tree.ItemFocused);
    TSI.Text:='New Item';
    Form := GetParentForm(DTree);
    if (Form <> nil) and (Form.Designer <> nil)
       then Form.Designer.Modified;
  end;
end;

procedure TItemsPropDlg.TreeItemFocused(Sender: TObject);
begin
  SubItemBtn.Enabled:=Tree.ItemFocused<>nil;
  EditBtn.Enabled:=Tree.ItemFocused<>nil;
  DeleteBtn.Enabled:=Tree.ItemFocused<>nil;
end;

procedure TItemsPropDlg.NewItemBtnClick(Sender: TObject);
var TSI:TBWTreeItem;
var Form : TCustomForm;
begin
  TSI:=Tree.Items.AddItem(nil);
  TSI.Text:='New Item';
  Form := GetParentForm(DTree);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TItemsPropDlg.EditBtnClick(Sender: TObject);
var T:TItemColDlg;
var Form : TCustomForm;
begin
  if Tree.ItemFocused=nil then exit;
  T:=nil;
  try
    T:=TItemColDlg.Create(self);
    T.Item:=Tree.ItemFocused;
    T.SetData;
    if T.ShowModal=mrOk then
    begin
      T.GetData;
      Form := GetParentForm(DTree);
      if (Form <> nil) and (Form.Designer <> nil)
         then Form.Designer.Modified;
    end;
  finally
    T.Free;
  end;
end;

procedure TItemsPropDlg.OKBtnClick(Sender: TObject);
begin
  DTree.Items:=Tree.Items;
end;

procedure TItemsPropDlg.SaveBtnClick(Sender: TObject);
var T:TFileStream;
begin
  if not SaveDlg.Execute then exit;
  T:=nil;
  try
    T:=TFileStream.Create(SaveDlg.FileName,fmCreate or fmShareDenyWrite);
    Tree.Items.SaveToStream(T);
  finally
    T.Free;
  end;
end;

procedure TItemsPropDlg.LoadBtnClick(Sender: TObject);
var T:TFileStream;
var Form : TCustomForm;
begin
  if not OpenDlg.Execute then exit;
  T:=nil;
  try
    T:=TFileStream.Create(OpenDlg.FileName,fmOpenRead or fmShareDenyWrite);
    Tree.Items.LoadFromStream(T);
    Form := GetParentForm(DTree);
    if (Form <> nil) and (Form.Designer <> nil)
       then Form.Designer.Modified;
  finally
    T.Free;
  end;
end;

procedure TItemsPropDlg.TreeStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  FDragItem:=Tree.ItemFocused;
  DragObject:=TBWDragObject.Create(Tree);
end;

procedure TItemsPropDlg.TreeDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
var TSI:TBWTreeItem;
begin
  Accept:=false;
  if Source<>Tree then exit;
  TSI:=Tree.GetItemAtY(Y);
  if (TSI<>nil) and (not TSI.IsUnder(FDragItem)) then Accept:=true;
end;

procedure TItemsPropDlg.TreeDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var TSI:TBWTreeItem;
var Form : TCustomForm;
begin
  TSI:=Tree.GetItemAtY(Y);
  if (TSI<>nil) and (not TSI.IsUnder(FDragItem))
     then FDragItem.MoveToIns(TSI.Parent, TSI.Index);
  Form := GetParentForm(DTree);
  if (Form <> nil) and (Form.Designer <> nil)
     then Form.Designer.Modified;
end;

procedure TItemsPropDlg.FormCreate(Sender: TObject);
begin
  TreeItemFocused(self);
end;

end.
